package ch.tvbauma.opencms.module.guestbook;

import com.opencms.boot.*;
import com.opencms.core.*;
import com.opencms.file.*;
import com.opencms.util.*;
import com.opencms.template.*;
import com.opencms.defaults.*;
import com.opencms.workplace.*;

import java.sql.*;
import java.text.*;
import java.util.*;

/**
 * <p>Title: Guestbook</p>
 * <p>Copyright: Freeware - NO WARRANTY</p>
 * <p>Company: Gymnastics Association Bauma, Switzerland</p>
 * @author Cyrill Brunschwiler - BRU
 * @version 1.0
 */

public class GuestbookContentDefinition
    extends A_CmsContentDefinition
    implements I_CmsContent {

  // Setup values of each guestbook entry
  private boolean m_isNewEntry = false;
  private String m_author = "";
  private String m_text = "";
  private String m_email = "";
  private String m_link = "";
  private long m_date = -1;
  private int m_lockstate = -1;
  private int m_id = -1;

  // Setup table name and property for id generation/storing
  private static final String C_TABLENAME = "guestbook_entry";
  private static final String C_TABLE_GUESTBOOK = "GUESTBOOK_ENTRY";
  public static final int C_NOT_LOCKED = -1;

  // Setup prepared statements for table manipulation
  private static final String C_INSERT_RECORD = "insert into " + C_TABLENAME +
      " values(?,?,?,?,?,?,?)";
  private static final String C_UPDATE_ENTRY = "update " + C_TABLENAME +
      " set author=?, text=?, email=?, link=?, lockstate=? where id=?";
  private static final String C_DELETE_RECORD = "delete from " + C_TABLENAME +
      " where id=?";
  private static final String C_SELECT_SINGLE = "select * from " + C_TABLENAME +
      " where id=? order by datetime desc";
  private static final String C_SELECT_PARTS = "select * from " + C_TABLENAME +
      " order by datetime desc limit ?,?";
  private static final String C_SELECT_ALL = "select * from " + C_TABLENAME +
      " order by datetime desc";
  private static final String C_COUNT = "select count(*) as nr_rows from " +
      C_TABLENAME;

  // Setup misc values
  private static Locale m_locale = new Locale("Default", "" + Locale.getDefault());
  private static String c_pool = null;
  private static String c_timeformat = null;
  private static String c_dateformat = null;

  /**
   * Init constructor
   */
  public GuestbookContentDefinition() {
    super();
  }

  public GuestbookContentDefinition(CmsObject cms, Integer id) throws CmsException {
    GuestbookContentDefinition gbcd = getEntry(id.intValue());

    m_id = gbcd.getId();
    m_author = gbcd.getAuthor();
    m_email = gbcd.getEmail();
    m_link = gbcd.getLink();
    m_text = gbcd.getText();
    m_date = gbcd.getDate();
    m_lockstate = gbcd.getLockstate();
  }

  /**
   * Create a new guestbook entry
   *
   * @param author The entry authors name
   * @param text The text written by the author
   * @param email The autors email address (optionally = "")
   * @param link A link pointing on autors ressources (optionally = "")
   * @throws CmsException if any error occurs while assigning a new id or while validating the input
   */
  public GuestbookContentDefinition(String author, String text, String email,
                                    String link) throws
      CmsException {

    this(com.opencms.dbpool.CmsIdGenerator.nextId(c_pool, C_TABLE_GUESTBOOK),
         author,
         text,
         email,
         link,
         new java.util.Date().getTime(),
         C_NOT_LOCKED);

    // indicate that this is a new element
    m_isNewEntry = true;
  }

  /**
   * Construct a guestbook entry
   *
   * @param id The elements id
   * @param author The entry authors name
   * @param text The text written by the author
   * @param email The autors email address (optionally = "")
   * @param link A link pointing on autors ressources (optionally = "")
   * @throws CmsException if any error occurs while assigning a new id or while validating the input
   */
  public GuestbookContentDefinition(int id, String author, String text,
                                    String email, String link,
                                    long date, int lockstate) {

    m_author = author;
    m_text = text;
    m_email = email;
    m_link = link;
    m_date = date;
    m_id = id;
    m_lockstate = lockstate;
  }

  public int getId() {
    return m_id;
  }

  public String getUniqueId(CmsObject cms) {
    return "" + m_id;
  }

  public String getAuthor() {
    return m_author;
  }

  public void setAuthor(String author) {
    m_author = author;
  }

  public String getText() {
    return m_text;
  }

  public void setText(String text) {
    m_text = text;
  }

  public String getEmail() {
    return m_email;
  }

  public void setEmail(String email) {
    m_email = email;
  }

  public String getLink() {
    return m_link;
  }

  public void setLink(String link) {
    m_link = link;
  }

  public long getDate() {
    return m_date;
  }

  /**
   * Sets the lockstate of the CD.
   *
   * @param lockstate The lockstate of this CD object.
   * (possibles states: locked by you,locked by other, unlocked)
   */
  public void setLockstate(int lockstate) {
    m_lockstate = lockstate;
  }

  /**
   * Gets the lockstate of the CD
   * (possibles states: locked by user,locked by other, unlocked)
   */
  public int getLockstate() {
    return m_lockstate;
  }

  /**
   * The content should be lockable so return true
   * @returns a boolean
   */
  public static boolean isLockable() {
    return true;
  }

  /**
   * Build my own date. Look @ constant DATE_FORMAT for details
   *
   * @return formatted date
   */
  public String getFormatedDate() {
    DateFormat sdf = new SimpleDateFormat(c_dateformat, m_locale);
    return sdf.format(new java.util.Date(m_date));
  }

  /**
   * Build my own time. Look @ constant TIME_FORMAT for details
   *
   * @return formatted time
   */
  public String getFormatedTime() {
    DateFormat sdf = new SimpleDateFormat(c_timeformat, m_locale);
    return sdf.format(new java.util.Date(m_date));
  }

  /**
   * Called by OpenCMS if any modul param changes
   *
   * @param cms The actual cms object
   * @throws CmsException if an error occurs while accessing the registry
   */
  public static void moduleParameterWasUpdated(CmsObject cms) throws
      CmsException {
    c_pool = OpenCms.getRegistry().getModuleParameterString(
        "ch.tvbauma.opencms.module.guestbook",
        "poolname");
    c_timeformat = OpenCms.getRegistry().getModuleParameterString(
        "ch.tvbauma.opencms.module.guestbook",
        "timeformat");
    c_dateformat = OpenCms.getRegistry().getModuleParameterString(
        "ch.tvbauma.opencms.module.guestbook",
        "dateformat");
  }

  /**
   * Write a new element into table
   *
   * @param cms An instance of CmsObject
   * @throws CmsException if an error occurs while writing data to DB
   */
  public void write(CmsObject cms) throws CmsException {

    Connection con = null;

    // clear the cache to make changes visible
    Util.clearCache(cms);

    /**
     * test if this is a new element and should be written to the table
     * otherwise the entry already exists and we just need to update
     */
    if (m_isNewEntry) {
      try {
        // write a record to database
        con = DriverManager.getConnection(c_pool);
        PreparedStatement pst = con.prepareStatement(C_INSERT_RECORD);
        pst.setInt(1, m_id);
        pst.setString(2, m_author);
        pst.setString(3, m_text);
        pst.setString(4, m_email);
        pst.setString(5, m_link);
        pst.setLong(6, new java.util.Date().getTime());
        pst.setInt(7, m_lockstate);
        pst.execute();
        pst.close();
        m_isNewEntry = false;
      }
      catch (Exception e) {
        throw new CmsException("[" + this.getClass().getName() + "] " +
                               e.getMessage(), CmsException.C_SQL_ERROR, e);
      }
      finally {
        try {
          con.close();
        }
        catch (Exception e) {
          throw new CmsException("[" + this.getClass().getName() + "] " +
                                 e.getMessage(), CmsException.C_SQL_ERROR, e);
        }
      }
    }
    else {
      // entry exsits allready in the table and we just need to update
      update(cms);
    }
  }

  /**
   * Deletes cms object (record) from table
   *
   * @param cms The object to delete
   * @throws CmsException if any error occurs during DB access
   */
  public void delete(CmsObject cms) throws CmsException {
    Connection con = null;

    // clear the cache to make changes visible
    Util.clearCache(cms);

    try {
      // delete a record from database
      con = DriverManager.getConnection(c_pool);
      PreparedStatement pst = con.prepareStatement(C_DELETE_RECORD);
      pst.setInt(1, m_id);
      pst.execute();
      pst.close();
    }
    catch (Exception e) {
      throw new CmsException("[" + this.getClass().getName() + "] " +
                             e.getMessage(), CmsException.C_SQL_ERROR, e);
    }
    finally {
      try {
        con.close();
      }
      catch (Exception e) {
        throw new CmsException("[" + this.getClass().getName() + "] " +
                               e.getMessage(), CmsException.C_SQL_ERROR, e);
      }
    }
  }

  /**
   * Method to updata an existing entry. This method is only called by
   * the method write()
   *
   * @param CmsObject The cms object
   * @throws CmsException if any db error occurs while updating th entry
   */
  private void update(CmsObject cms) throws CmsException {

    Connection con = null;
    PreparedStatement statement = null;

    try {
      // write the data to database
      con = DriverManager.getConnection(c_pool);
      statement = con.prepareStatement(C_UPDATE_ENTRY);
      statement.setString(1, m_author);
      statement.setString(2, m_text);
      statement.setString(3, m_email);
      statement.setString(4, m_link);
      statement.setInt(5, m_lockstate);
      statement.setInt(6, m_id);
      statement.executeUpdate();
    }
    catch (Exception e) {
      throw new CmsException("[" + this.getClass().getName() + "] " +
                             e.getMessage(),
                             CmsException.C_SQL_ERROR, e);
    }
    finally {
      try {
        statement.close();
      }
      catch (Exception e) {
        // ignore the exception
      }
      try {
        con.close();
      }
      catch (Exception e) {
        // ignore the exception
      }
    }
  }

  /**
   * Gets some records from the table. eg. 10-13
   *
   * @param from The row number of the first record in the part
   * @param limit The max number of records
   * @return a vector filled with the requested content objects
   * @throws CmsException if any error occurs during db access
   */
  public static Vector getEntries(int from, int limit) throws CmsException {
    Connection con = null;
    ResultSet rs = null;
    Vector vecEntries = new Vector();

    try {
      con = DriverManager.getConnection(c_pool);
      PreparedStatement pst = con.prepareStatement(C_SELECT_PARTS);
      pst.setInt(1, from);
      pst.setInt(2, limit);
      rs = pst.executeQuery();
      pst.close();

      while (rs.next()) {
        vecEntries.add(new GuestbookContentDefinition(rs.getInt(1),
            rs.getString(2),
            rs.getString(3),
            rs.getString(4),
            rs.getString(5),
            rs.getLong(6),
            rs.getInt(7)));
      }
    }
    catch (Exception e) {
      throw new CmsException("[GuestbookContentDefinition] " + e.getMessage(),
                             CmsException.C_SQL_ERROR, e);
    }
    finally {
      try {
        con.close();
      }
      catch (Exception e) {
        throw new CmsException("[GuestbookContentDefinition] " + e.getMessage(),
                               CmsException.C_SQL_ERROR, e);
      }
    }
    return vecEntries;
  }

  /**
   * Gets an entry from the table
   *
   * @param id The id of the entry
   * @return a content definition with the specified entry if found otherwise null
   * @throws CmsException if any error occurs during db access
   */
  public static GuestbookContentDefinition getEntry(int id) throws CmsException {
    Connection con = null;
    ResultSet rs = null;
    Vector vecEntries = new Vector();
    GuestbookContentDefinition gbcd = null;

    try {
      con = DriverManager.getConnection(c_pool);
      PreparedStatement pst = con.prepareStatement(C_SELECT_SINGLE);
      pst.setInt(1, id);
      rs = pst.executeQuery();
      pst.close();

      while (rs.next()) {
        gbcd = new GuestbookContentDefinition(rs.getInt(1),
                                              rs.getString(2),
                                              rs.getString(3),
                                              rs.getString(4),
                                              rs.getString(5),
                                              rs.getLong(6),
                                              rs.getInt(7));
      }
    }
    catch (Exception e) {
      throw new CmsException("[GuestbookContentDefinition] " + e.getMessage(),
                             CmsException.C_SQL_ERROR, e);
    }
    finally {
      try {
        con.close();
      }
      catch (Exception e) {
        throw new CmsException("[GuestbookContentDefinition] " + e.getMessage(),
                               CmsException.C_SQL_ERROR, e);
      }
    }

    return gbcd;
  }

  /**
   * Gets all records from the table.
   *
   * @return a vector cotaining all content objects
   * @throws CmsException if any error occurs during db access
   */
  public static Vector getAllEntries() throws CmsException {
    Connection con = null;
    ResultSet rs = null;
    Vector vecEntries = new Vector();

    try {
      con = DriverManager.getConnection(c_pool);
      PreparedStatement pst = con.prepareStatement(C_SELECT_ALL);
      rs = pst.executeQuery();
      pst.close();

      while (rs.next()) {
        vecEntries.add(new GuestbookContentDefinition(rs.getInt(1),
            rs.getString(2),
            rs.getString(3),
            rs.getString(4),
            rs.getString(5),
            rs.getLong(6),
            rs.getInt(7)));
      }
    }
    catch (Exception e) {
      throw new CmsException("[GuestbookContentDefinition] " + e.getMessage(),
                             CmsException.C_SQL_ERROR, e);
    }
    finally {
      try {
        con.close();
      }
      catch (Exception e) {
        throw new CmsException("[GuestbookContentDefinition] " + e.getMessage(),
                               CmsException.C_SQL_ERROR, e);
      }
    }
    return vecEntries;
  }

  /**
   * Counts the records in the table.
   *
   * @return The number of records or 0 if an error occured
   * @throws CmsException if any error occurs during db access
   */
  public static int getNrRecords() throws CmsException {
    Connection con = null;
    ResultSet rs = null;

    try {
      con = DriverManager.getConnection(c_pool);
      PreparedStatement pst = con.prepareStatement(C_COUNT);
      rs = pst.executeQuery();
      pst.close();

      if (rs.next())
        return rs.getInt(1);
      else
        return 0;
    }
    catch (Exception e) {
      throw new CmsException("[GuestbookContentDefinition] " + e.getMessage(),
                             CmsException.C_SQL_ERROR, e);
    }
    finally {
      try {
        con.close();
      }
      catch (Exception e) {
        throw new CmsException("[GuestbookContentDefinition] " + e.getMessage(),
                               CmsException.C_SQL_ERROR, e);
      }
    }
  }

  /**
   * Declaration of the methods to get the fields. Methode used by backoffice.
   *
   * @param cms The cms object
   * @return a Vector containing the methods
   */
  public static Vector getFieldMethods(CmsObject cms) {

    Vector methods = new Vector();

    try {
      methods.addElement(GuestbookContentDefinition.class.getMethod("getFormatedDate",
          new Class[0]));
      methods.addElement(GuestbookContentDefinition.class.getMethod("getFormatedTime",
          new Class[0]));
      methods.addElement(GuestbookContentDefinition.class.getMethod("getAuthor",
          new Class[0]));
      methods.addElement(GuestbookContentDefinition.class.getMethod("getText",
          new Class[0]));
      methods.addElement(GuestbookContentDefinition.class.getMethod("getEmail",
          new Class[0]));
      methods.addElement(GuestbookContentDefinition.class.getMethod("getLink",
          new Class[0]));
    }
    catch (NoSuchMethodException e) {
      System.err.println(
          "Exception in GuestbookContentDefinition.getFieldMethods(CmsObject)!" +
          e.getMessage());
    }

    return methods;
  }

  /**
   * Declaration of the fieldnames to display by the backoffice
   *
   * @param cms The cms object
   * @return a Vector conataining the field names as strings
   */
  public static Vector getFieldNames(CmsObject cms) {
    Vector names = new Vector();

    names.addElement("Date");
    names.addElement("Time");
    names.addElement("Author");
    names.addElement("Text");
    names.addElement("Email");
    names.addElement("Link");

    return names;
  }

  /**
   * Just display default filter
   *
   * @return An empty vector
   */
  public static Vector getFilterMethods(CmsObject cms) {

    Vector filterMethods = new Vector();
    String showText = "";

    // add the always shown first filterMethod...
    try {
      CmsXmlLanguageFile lang = new CmsXmlLanguageFile(cms);
      String prefix = "ch_tvbauma_opencms_module_guestbook_GuestbookContentDefiniton.label.filter.";

      filterMethods.addElement(new CmsFilterMethod(lang.getLanguageValue(prefix + "default.title"),
          GuestbookContentDefinition.class.getMethod(lang.getLanguageValue(prefix + "default.method"),
          new Class[] {String.class}),
          new Object[] {}));
      filterMethods.addElement(new CmsFilterMethod(lang.getLanguageValue(prefix + "author.title"),
          GuestbookContentDefinition.class.getMethod(lang.getLanguageValue(prefix + "author.method"),
          new Class[] {String.class}),
          new Object[] {}));
      filterMethods.addElement(new CmsFilterMethod(lang.getLanguageValue(prefix + "text.title"),
          GuestbookContentDefinition.class.getMethod(lang.getLanguageValue(prefix + "text.method"),
          new Class[] {String.class}),
          new Object[] {}));
    }
    catch (NoSuchMethodException e) {
      if (I_CmsLogChannels.C_PREPROCESSOR_IS_LOGGING && A_OpenCms.isLogging()) {
        A_OpenCms.log("opencms_info",
                      "NewsContentDefinition: NewsContentDefinition getFilterMethods: Exception in NewsContentDefinition.getFilterMethods(CmsObject)!" +
                      e.getMessage());
      }
      System.err.println(
          "Exception in GuestbookContentDefinition.getFilterMethods(CmsObject)!" +
          e.getMessage());
    }
    catch (CmsException cmse) {
      if (I_CmsLogChannels.C_PREPROCESSOR_IS_LOGGING && A_OpenCms.isLogging()) {
        A_OpenCms.log("opencms_info",
                      "GuestbookContentDefinition.getFilterMethods(CmsObject)! Error getting props" +
                      cmse.getMessage());
      }
      System.err.println(
          "Exception in GuestbookContentDefinition.getFilterMethods(CmsObject)! Error getting props" +
          cmse.getMessage());
    }

    return filterMethods;
  }

  /**
   * Default filter (just sorts for date)
   * this method creates a list of all enties
   *
   * @return a Vector with all Elementes
   */
  public static Vector getSortedList(String str) {

    // setup values
    Connection con = null;
    PreparedStatement pstmt = null;
    ResultSet rs = null;
    Vector list = new Vector();

    // build statement, execute query and fill vector (default sorted)
    try {
      // get connection from pool
      con = DriverManager.getConnection(c_pool);
      pstmt = con.prepareStatement(C_SELECT_ALL);
      rs = pstmt.executeQuery();

      while (rs.next()) {
        list.add(new GuestbookContentDefinition(rs.getInt(1),
                                                rs.getString(2),
                                                rs.getString(3),
                                                rs.getString(4),
                                                rs.getString(5),
                                                rs.getLong(6),
                                                rs.getInt(7)));
      }
    }
    catch (Exception e) {
      System.err.println(
          "[GuestbookContentDefinition.getSortedList()] Exception: " +
          e.getMessage());
    }
    finally {
      try {
        rs.close();
      }
      catch (Exception e) {
        // ignore the exception
      }
      try {
        pstmt.close();
      }
      catch (Exception e) {
        // ignore the exception
      }
      try {
        con.close();
      }
      catch (Exception e) {
        // ignore the exception
      }
    }
    return list;
  }

  /**
   * Filter for author name
   * this method creates a list of the matched entries
   *
   * @return a Vector with all Elementes
   */
  public static Vector getByAuthor(String str) {

    // setup values
    Connection con = null;
    PreparedStatement pstmt = null;
    ResultSet rs = null;
    Vector list = new Vector();

    // input validation
    if (str == null || str == " ") {
      str = "";
    }

    str.trim();
    str.toLowerCase();

    // build statement, execute query and fill vector (default sorted)
    try {
      // get connection from pool
      con = DriverManager.getConnection(c_pool);
      pstmt = con.prepareStatement(C_SELECT_ALL);
      rs = pstmt.executeQuery();

      while (rs.next()) {

        String author = rs.getString(2);

        if (author.indexOf(str) != -1)
          list.add(new GuestbookContentDefinition(rs.getInt(1),
                                                  rs.getString(2),
                                                  rs.getString(3),
                                                  rs.getString(4),
                                                  rs.getString(5),
                                                  rs.getLong(6),
                                                  rs.getInt(7)));
      }
    }
    catch (Exception e) {
      System.err.println(
          "[GuestbookContentDefinition.getSortedList()] Exception: " +
          e.getMessage());
    }
    finally {
      try {
        rs.close();
      }
      catch (Exception e) {
        // ignore the exception
      }
      try {
        pstmt.close();
      }
      catch (Exception e) {
        // ignore the exception
      }
      try {
        con.close();
      }
      catch (Exception e) {
        // ignore the exception
      }
    }
    return list;
  }

  /**
   * Filter for text
   * this method creates a list of the matched entries
   *
   * @return a Vector with all Elementes
   */
  public static Vector getByText(String str) {

    // setup values
    Connection con = null;
    PreparedStatement pstmt = null;
    ResultSet rs = null;
    Vector list = new Vector();

    // input validation
    if (str == null || str == " ") {
      str = "";
    }

    str.trim();
    str.toLowerCase();

    // build statement, execute query and fill vector (default sorted)
    try {
      // get connection from pool
      con = DriverManager.getConnection(c_pool);
      pstmt = con.prepareStatement(C_SELECT_ALL);
      rs = pstmt.executeQuery();

      while (rs.next()) {

        String text = rs.getString(3);

        if (text.indexOf(str) != -1)
          list.add(new GuestbookContentDefinition(rs.getInt(1),
                                                  rs.getString(2),
                                                  rs.getString(3),
                                                  rs.getString(4),
                                                  rs.getString(5),
                                                  rs.getLong(6),
                                                  rs.getInt(7)));
      }
    }
    catch (Exception e) {
      System.err.println(
          "[GuestbookContentDefinition.getSortedList()] Exception: " +
          e.getMessage());
    }
    finally {
      try {
        rs.close();
      }
      catch (Exception e) {
        // ignore the exception
      }
      try {
        pstmt.close();
      }
      catch (Exception e) {
        // ignore the exception
      }
      try {
        con.close();
      }
      catch (Exception e) {
        // ignore the exception
      }
    }
    return list;
  }

  /**
   * Just to get the actual date as a formated string. Set the format of the
   * date in the public constant DATE_FORMAT. Look up the JDK doc for possible
   * values.
   *
   * @return The formated string
   */
  public static String getActualDate() {
    DateFormat sdf = new SimpleDateFormat(GuestbookContentDefinition.
                                          c_dateformat, m_locale);
    return sdf.format(new java.util.Date());
  }

  /**
   * Just to get the actual time as a formated string. Set the format of the
   * time in the public constant TIME_FORMAT. Look up the JDK doc for possible
   * values.
   *
   * @return The formated string
   */
  public static String getActualTime() {
    DateFormat sdf = new SimpleDateFormat(GuestbookContentDefinition.
                                          c_timeformat, m_locale);
    return sdf.format(new java.util.Date());
  }

  /**
   *  This code is executed on load-time of this class
   */
  static {
    try {
      // set the pool
      moduleParameterWasUpdated(null);
    }
    catch (CmsException exc) {
      System.err.println("[GuestbookContentDefinition]: error loading pool.");
    }
  }
}