package ch.tvbauma.opencms.module.guestbook;

import com.opencms.workplace.*;
import com.opencms.core.*;
import com.opencms.file.*;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import java.util.regex.*;
import javax.servlet.http.*;

/**
 * <p>Title: Guestbook</p>
 * <p>Copyright: Freeware - NO WARRANTY</p>
 * <p>Company: Gymnastics Association Bauma, Switzerland</p>
 * @author Cyrill Brunschwiler - BRU
 * @version 1.0
 */

public class Util {

  private static final String s_prefix =
      "ch_tvbauma_opencms_module_guestbook_GalleryUtil.";

  /**
   * To check if a text matches a pattern.
   *
   * @param text The text to test
   * @param pattern The pattern which is used to test the text for
   * @return true if successfully matched otherwise false
   * @throws Exception if an error occures while compiling pattern
   */
  public static boolean check(String text, String pattern) throws Exception {

    Pattern p = Pattern.compile(pattern);
    Matcher m = p.matcher(text);

    return m.matches();
  }

  /**
   * To check wether a smiley with the specified name exists or not.
   *
   * @param name The name of the smiley (gets attached the type. eg '.gif'
   * @param cms The actual cms object
   * @return true if such a smiley exists otherwise false
   * @throws Exception if an error occurs while loading type from properties
   */
  public static boolean checkForSmiley(String name, CmsObject cms) throws
      Exception {
    CmsXmlLanguageFile lang = new CmsXmlLanguageFile(cms);
    String pattern = lang.getLanguageValue(s_prefix + "pattern.img");
    String type = lang.getLanguageValue(s_prefix + "type.img");

    name = name.concat(type);

    // check if the name is correct
    if (check(name, pattern)) {
      // name ok... try to get the resource
      CmsResource res = getImageResource(cms, "guestbook", name);

      // check if we were able to get the resource
      if (res != null)
        return true;
    }

    return false;
  }

  /**
   * Method to clear the cache
   *
   * @param cms the actual object
   */
  public static void clearCache(CmsObject cms) {

    // first the element cache
    cms.getOnlineElementCache().clearCache();

    URL u = null;
    String p = ( (HttpServletRequest)
                cms.getRequestContext().getRequest().getOriginalRequest()).
        getServletPath();
    String s = ( (HttpServletRequest)
                cms.getRequestContext().getRequest().getOriginalRequest()).
        getServerName();
    try {
      u = new URL("http://" + s + p +
                  "/system/workplace/action/login.html?_clearcache=all");
      URLConnection uc = u.openConnection();
      uc.connect();
      InputStream in = uc.getInputStream();
    }
    catch (MalformedURLException e) {
      System.err.println("Exception1: " + e.getMessage());
    }
    catch (IOException e) {
      System.err.println("Exception2: " + e.getMessage());
    }
  }

  /**
   * Filter a Vector for the smiley resources by pattern
   *
   * @param vecAll The Vector containig mixed images (thumbs, originals)
   * @param cms The actual cms object
   * @return A Vector just containing the original resources
   * @throws Exception either if it can't compile the pattern property or if it
   *                   can't load the property from file
   */
  public static Vector getSmileys(Vector vecAll, CmsObject cms) throws
      Exception {

    CmsXmlLanguageFile lang = new CmsXmlLanguageFile(cms);
    String pattern = lang.getLanguageValue(s_prefix + "pattern.img");

    Iterator it = vecAll.iterator();

    while (it.hasNext()) {
      CmsResource res = (CmsResource) it.next();

      String name = res.getName();

      if (!Util.check(name, pattern))
        it.remove();
    }

    return vecAll;
  }

  /**
   * Get all resources (files) from a gallery folder by gallery name
   *
   * @param cms The CmsObject
   * @param name The name of the picture gallery
   * @return A Vector containing the requested files or an empty vector
   * @throws CmsException if any error occurs
   */
  public static Vector getResourcesFromFolder(CmsObject cms, String name) throws
      CmsException {

    Vector vecFiles = cms.getFilesInFolder(I_CmsWpConstants.C_VFS_GALLERY_PICS +
                                           name);

    return vecFiles;
  }

  /**
   * Get the title property of a resource
   *
   * @param cms The CmsObject
   * @param res The resource to gather the title from
   * @return The title of the resource or null if it wasn't available
   */
  public static String getTitle(CmsObject cms, CmsResource res) throws
      CmsException {
    return cms.readProperty(res.getAbsolutePath(),
                            I_CmsConstants.C_PROPERTY_TITLE);
  }

  /**
   * Get an image from a specific picgallery folder
   *
   * @param cms The actual cms object
   * @param gallery The name of the gallery to get the picture from
   * @param image The name of the image to get
   * @return The picture as CmsResource if success otherwise null
   * @throws CmsException if an error occurs while getting resources from cms
   */
  public static CmsResource getImageResource(CmsObject cms, String gallery,
                                             String image) throws CmsException {

    Vector vecImages = getResourcesFromFolder(cms, gallery);
    Iterator it = vecImages.iterator();

    while (it.hasNext()) {
      CmsResource res = (CmsResource) it.next();

      if (res.getName().equals(image))
        return res;
    }

    return null;
  }

  /**
   * Replaces all possible smiley tags to a real html image tag.
   * Wrong or empty smiley tags will be replaced with "".
   *
   * Eg. Hello World[laugh] -> HelloWorld<img src="my/defined/export/path/guestbook/laugh.gif">
   *
   * @param text The text to replace
   * @param cms The actual cms object
   * @return The replaced String
   * @throws Exception if it can't read properties or can't compile patterns
   */
  public static String replaceSmileyTags(String text, String path, CmsObject cms) throws
      Exception {

    CmsXmlLanguageFile lang = new CmsXmlLanguageFile(cms);
    String patternTag = lang.getLanguageValue(s_prefix + "pattern.tag");
    String patternName = lang.getLanguageValue(s_prefix + "pattern.name");
    String type = lang.getLanguageValue(s_prefix + "type.img");

    Pattern pName = Pattern.compile(patternName);
    Pattern pTag = Pattern.compile(patternTag);

    Matcher mTag = pTag.matcher(text);

    // do tag replacement while we find some smiley tags
    while (mTag.find()) {

        // get first part which maches
        String partTag = mTag.group(0);

        // get name from tag
        Matcher mName = pName.matcher(partTag);

        if (mName.find()) {
          String partName = mName.group();

          if (checkForSmiley(partName, cms)) {
            // replace first smiley tag with real html image tag
            text = mTag.replaceFirst(" <img src=\"" + path + "guestbook/" +
                                     partName + type + "\"> ");
            mTag.reset(text);
          }
          else {
            // clear wrong tag
            text = mTag.replaceFirst("");
          }
      }
    }
    return text;
  }

  /**
   * Check if all tags, which look like smiley tags realy are available smileys
   *
   * @param text The text to check
   * @param cms The actual cms object
   * @return true if only available smiley tags were found otherwise false
   * @throws Exception if it can't read properties or can't compile patterns
   */
  public static boolean checkSmileyTags(String text, CmsObject cms) throws
      Exception {

    CmsXmlLanguageFile lang = new CmsXmlLanguageFile(cms);
    String patternTag = lang.getLanguageValue(s_prefix + "pattern.tag");
    String patternName = lang.getLanguageValue(s_prefix + "pattern.name");
    String type = lang.getLanguageValue(s_prefix + "type.img");

    Pattern pName = Pattern.compile(patternName);
    Pattern pTag = Pattern.compile(patternTag);

    Matcher mTag = pTag.matcher(text);

    while (mTag.find()) {
      String tag = mTag.group(0);
      Matcher mName = pName.matcher(tag);

      // get name from tag and check wether its an existing smiley
      if (mName.find()) {
        if (!checkForSmiley(mName.group(0), cms))
          return false;
      }
    }

    // if it reaches this row, it didn't find any wrong smiley tags
    return true;
  }

  /**
   * Builds a String with the resources from the guestbook folder
   *
   * @param cms The actual cms object
       * @return The string containing the image tags or null if no images were found
   * @throws Exception if it can't load the properties
   */
  public static String getAllSmileysAsHTML(CmsObject cms, String path) throws Exception {

    CmsXmlLanguageFile lang = new CmsXmlLanguageFile(cms);
    String type = lang.getLanguageValue(s_prefix + "type.img");

    Vector vecAll = getResourcesFromFolder(cms, "guestbook");
    String html = "";

    if (vecAll != null) {
      Vector vecSmileys = getSmileys(vecAll, cms);
      Iterator it = vecSmileys.iterator();

      while (it.hasNext()) {
        CmsResource res = (CmsResource) it.next();
        String name = res.getName();

        html += "<a href=\"#\" onclick=\"insertsmiley(document.getElementsByName('text')[0],'[" +
            name.substring(0, name.length() - type.length()) +
            "]')\"><img src=\"" + path + "guestbook/" + name +
            "\" border=\"0\" alt=\"[" +
            name.substring(0, name.length() - type.length()) + "]\"></a> ";
      }

      return html;
    }

    return null;
  }
}